@@ -31,10 +31,11 @@
  *  -l                     print interesting OpenGL limits (added 5 Sep 2002)
  */
 
-#include <windows.h>
 
-#include <GL/glew.h>
-#include <GL/wglew.h>
+#include <windows.h>
+#include <stdbool.h>
+#include <glad/glad.h>
+#include <glad/glad_wgl.h>
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
@@ -42,14 +43,6 @@
 #include "glinfo_common.h"
 
 
-static GLboolean have_WGL_ARB_create_context;
-static GLboolean have_WGL_ARB_pixel_format;
-static GLboolean have_WGL_ARB_multisample;
-
-static PFNWGLGETPIXELFORMATATTRIBIVARBPROC wglGetPixelFormatAttribivARB_func;
-static PFNWGLCREATECONTEXTATTRIBSARBPROC wglCreateContextAttribsARB_func;
-
-
 /**
  * An extension of PIXELFORMATDESCRIPTOR to handle multisample, etc.
  */
@@ -57,6 +50,11 @@ struct format_info {
    PIXELFORMATDESCRIPTOR pfd;
    int sampleBuffers, numSamples;
    int transparency;
+   bool floatComponents;
+   bool srgb;
+   bool draw_to_bitmap;
+   bool draw_to_pbuffer;
+   bool gdi_drawing;
 };
 
 
@@ -152,43 +150,21 @@ print_screen_info(HDC _hdc, const struct options *opts, GLboolean coreProfile)
    }
 
    if (wglMakeCurrent(hdc, ctx)) {
-#if defined(WGL_ARB_extensions_string)
-      PFNWGLGETEXTENSIONSSTRINGARBPROC wglGetExtensionsStringARB_func = 
-         (PFNWGLGETEXTENSIONSSTRINGARBPROC)wglGetProcAddress("wglGetExtensionsStringARB");
-#endif
+      gladLoadGL();
+      gladLoadWGL(hdc);
       const char *glVendor, *glRenderer, *glVersion, *glExtensions;
       const char *wglExtensions = NULL;
       struct ext_functions extfuncs;
       
-#if defined(WGL_ARB_extensions_string)
-      if (wglGetExtensionsStringARB_func) {
-         wglExtensions = wglGetExtensionsStringARB_func(hdc);
-         if (extension_supported("WGL_ARB_pixel_format", wglExtensions)) {
-            have_WGL_ARB_pixel_format = GL_TRUE;
-         }
-         if (extension_supported("WGL_ARB_multisample", wglExtensions)) {
-            have_WGL_ARB_multisample = GL_TRUE;
-         }
-         if (extension_supported("WGL_ARB_create_context", wglExtensions)) {
-            have_WGL_ARB_create_context = GL_TRUE;
-         }
+      if (GLAD_WGL_ARB_extensions_string) {
+         wglExtensions = wglGetExtensionsStringARB(hdc);
       }
-#endif
 
-      if (coreProfile && have_WGL_ARB_create_context) {
+      if (coreProfile && GLAD_WGL_ARB_create_context) {
          /* Try to create a new, core context */
          HGLRC core_ctx = 0;
          int i;
 
-         wglCreateContextAttribsARB_func =
-            (PFNWGLCREATECONTEXTATTRIBSARBPROC)
-            wglGetProcAddress("wglCreateContextAttribsARB");
-         assert(wglCreateContextAttribsARB_func);
-         if (!wglCreateContextAttribsARB_func) {
-            printf("Failed to get wglCreateContextAttribsARB pointer.");
-            return;
-         }
-
          for (i = 0; gl_versions[i].major > 0; i++) {
             int attribs[10], n;
 
@@ -208,7 +184,7 @@ print_screen_info(HDC _hdc, const struct options *opts, GLboolean coreProfile)
             }
             attribs[n++] = 0;
 
-            core_ctx = wglCreateContextAttribsARB_func(hdc, 0, attribs);
+            core_ctx = wglCreateContextAttribsARB(hdc, 0, attribs);
             if (core_ctx) {
                break;
             }
@@ -230,12 +206,9 @@ print_screen_info(HDC _hdc, const struct options *opts, GLboolean coreProfile)
          coreProfile = GL_FALSE;
       }
 
-      extfuncs.GetProgramivARB = (PFNGLGETPROGRAMIVARBPROC)
-         wglGetProcAddress("glGetProgramivARB");
-      extfuncs.GetStringi = (PFNGLGETSTRINGIPROC)
-         wglGetProcAddress("glGetStringi");
-      extfuncs.GetConvolutionParameteriv = (GETCONVOLUTIONPARAMETERIVPROC)
-         wglGetProcAddress("glGetConvolutionParameteriv");
+      extfuncs.GetProgramivARB = glGetProgramivARB;
+      extfuncs.GetStringi = glGetStringi;
+      extfuncs.GetConvolutionParameteriv = glGetConvolutionParameteriv;
 
       glVendor = (const char *) glGetString(GL_VENDOR);
       glRenderer = (const char *) glGetString(GL_RENDERER);
@@ -264,26 +237,20 @@ print_screen_info(HDC _hdc, const struct options *opts, GLboolean coreProfile)
 
       version = (glVersion[0] - '0') * 10 + (glVersion[2] - '0');
 
-#ifdef GL_VERSION_2_0
-      if (version >= 20) {
+      if (GLAD_GL_VERSION_2_0) {
          char *v = (char *) glGetString(GL_SHADING_LANGUAGE_VERSION);
          printf("%s shading language version string: %s\n", oglString, v);
       }
-#endif
-#ifdef GL_VERSION_3_0
-      if (version >= 30) {
+      if (GLAD_GL_VERSION_3_0) {
          GLint flags;
          glGetIntegerv(GL_CONTEXT_FLAGS, &flags);
          printf("%s context flags: %s\n", oglString, context_flags_string(flags));
       }
-#endif
-#ifdef GL_VERSION_3_2
-      if (version >= 32) {
+      if (GLAD_GL_VERSION_3_2) {
          GLint mask;
          glGetIntegerv(GL_CONTEXT_PROFILE_MASK, &mask);
          printf("%s profile mask: %s\n", oglString, profile_mask_string(mask));
       }
-#endif
 
       if (opts->mode != Brief) {
          printf("%s extensions:\n", oglString);
@@ -318,18 +285,23 @@ visual_render_type_name(BYTE iPixelType)
 static void
 print_visual_attribs_verbose(int iPixelFormat, const struct format_info *info)
 {
-   printf("Visual ID: %x  generic=%d  native=%d\n",
+   printf("Visual ID: %x generic=%d drawToWindow=%d drawToBitmap=%d drawToPBuffer=%d GDI=%d\n",
           iPixelFormat, 
           info->pfd.dwFlags & PFD_GENERIC_FORMAT ? 1 : 0,
-          info->pfd.dwFlags & PFD_DRAW_TO_WINDOW ? 1 : 0);
+          info->pfd.dwFlags & PFD_DRAW_TO_WINDOW ? 1 : 0,
+          info->draw_to_bitmap,
+          info->draw_to_pbuffer,
+          info->gdi_drawing);
    printf("    bufferSize=%d level=%d renderType=%s doubleBuffer=%d stereo=%d\n",
           0 /* info->pfd.bufferSize */, 0 /* info->pfd.level */,
 	  visual_render_type_name(info->pfd.iPixelType),
           info->pfd.dwFlags & PFD_DOUBLEBUFFER ? 1 : 0, 
           info->pfd.dwFlags & PFD_STEREO ? 1 : 0);
-   printf("    rgba: cRedBits=%d cGreenBits=%d cBlueBits=%d cAlphaBits=%d\n",
+   printf("    rgba: cRedBits=%d cGreenBits=%d cBlueBits=%d cAlphaBits=%d float=%c sRGB=%c\n",
           info->pfd.cRedBits, info->pfd.cGreenBits,
-          info->pfd.cBlueBits, info->pfd.cAlphaBits);
+          info->pfd.cBlueBits, info->pfd.cAlphaBits,
+          info->floatComponents ? 'Y' : 'N',
+          info->srgb ? 'Y' : 'N');
    printf("    cAuxBuffers=%d cDepthBits=%d cStencilBits=%d\n",
           info->pfd.cAuxBuffers, info->pfd.cDepthBits, info->pfd.cStencilBits);
    printf("    accum: cRedBits=%d cGreenBits=%d cBlueBits=%d cAlphaBits=%d\n",
@@ -343,25 +315,22 @@ print_visual_attribs_verbose(int iPixelFormat, const struct format_info *info)
       printf("    swapMethod = Copy\n");
    else
       printf("    swapMethod = Undefined\n");
-
 }
 
 
 static void
 print_visual_attribs_short_header(void)
 {
-   printf("   visual   x   bf lv rg d st colorbuffer ax dp st accumbuffer  ms  cav\n");
-   printf(" id gen win sp  sz l  ci b ro  r  g  b  a bf th cl  r  g  b  a ns b eat\n");
-   printf("------------------------------------------------------------------------\n");
+   printf("    visual   x   bf lv rg d st colorbuffer   sr ax dp st accumbuffer  ms \n");
+   printf("  id gen win sp  sz l  ci b ro  r  g  b  a F gb bf th cl  r  g  b  a ns b\n");
+   printf("-------------------------------------------------------------------------\n");
 }
 
 
 static void
 print_visual_attribs_short(int iPixelFormat, const struct format_info *info)
 {
-   char *caveat = "None";
-
-   printf("0x%02x %2d  %2d %2d %3d %2d %c%c %c  %c %2d %2d %2d %2d %2d %2d %2d",
+   printf("0x%03x %2d  %2d %2d %3d %2d %c%c %c  %c %2d %2d %2d %2d %c  %c %2d %2d %2d",
           iPixelFormat,
           info->pfd.dwFlags & PFD_GENERIC_FORMAT ? 1 : 0,
           info->pfd.dwFlags & PFD_DRAW_TO_WINDOW ? 1 : 0,
@@ -374,33 +343,33 @@ print_visual_attribs_short(int iPixelFormat, const struct format_info *info)
           info->pfd.dwFlags & PFD_STEREO ? 'y' : '.',
           info->pfd.cRedBits, info->pfd.cGreenBits,
           info->pfd.cBlueBits, info->pfd.cAlphaBits,
+          info->floatComponents ? 'f' : '.',
+          info->srgb ? 's' : '.',
           info->pfd.cAuxBuffers,
           info->pfd.cDepthBits,
           info->pfd.cStencilBits
           );
 
-   printf(" %2d %2d %2d %2d %2d %1d %s\n",
+   printf(" %2d %2d %2d %2d %2d %1d\n",
           info->pfd.cAccumRedBits, info->pfd.cAccumGreenBits,
           info->pfd.cAccumBlueBits, info->pfd.cAccumAlphaBits,
-          info->numSamples, info->sampleBuffers,
-          caveat
-          );
+          info->numSamples, info->sampleBuffers);
 }
 
 
 static void
 print_visual_attribs_long_header(void)
 {
- printf("Vis  Vis   Visual Trans  buff lev render DB ste  r   g   b   a  aux dep ste  accum buffers  MS   MS\n");
- printf(" ID Depth   Type  parent size el   type     reo sz  sz  sz  sz  buf th  ncl  r   g   b   a  num bufs\n");
- printf("----------------------------------------------------------------------------------------------------\n");
+ printf("Vis   Vis   Visual Trans  buff lev render DB ste  r   g   b   a      s  aux dep ste  accum buffers  MS   MS \n");
+ printf(" ID  Depth   Type  parent size el   type     reo sz  sz  sz  sz flt rgb buf th  ncl  r   g   b   a  num bufs\n");
+ printf("------------------------------------------------------------------------------------------------------------\n");
 }
 
 
 static void
 print_visual_attribs_long(int iPixelFormat, const struct format_info *info)
 {
-   printf("0x%2x %2d %11d %2d     %2d %2d  %4s %3d %3d %3d %3d %3d %3d",
+   printf("0x%3x %2d %11d %2d     %2d %2d  %4s %3d %3d %3d %3d %3d %3d",
           iPixelFormat,
           info->pfd.dwFlags & PFD_GENERIC_FORMAT ? 1 : 0,
           info->pfd.dwFlags & PFD_DRAW_TO_WINDOW ? 1 : 0,
@@ -414,7 +383,9 @@ print_visual_attribs_long(int iPixelFormat, const struct format_info *info)
           info->pfd.cBlueBits, info->pfd.cAlphaBits
           );
 
-   printf(" %3d %4d %2d %3d %3d %3d %3d  %2d  %2d\n",
+   printf("  %c   %c %3d %4d %2d %3d %3d %3d %3d  %2d  %2d\n",
+          info->floatComponents ? 'f' : '.',
+          info->srgb ? 's' : '.',
           info->pfd.cAuxBuffers,
           info->pfd.cDepthBits,
           info->pfd.cStencilBits,
@@ -434,8 +405,8 @@ static int
 get_pf_attrib(HDC hdc, int pf, int attrib)
 {
    int layer = 0, value;
-   assert(have_WGL_ARB_pixel_format);
-   if (wglGetPixelFormatAttribivARB_func(hdc, pf, layer, 1, &attrib, &value)) {
+   assert(GLAD_WGL_ARB_pixel_format);
+   if (wglGetPixelFormatAttribivARB(hdc, pf, layer, 1, &attrib, &value)) {
       return value;
    }
    else {
@@ -452,7 +423,7 @@ get_format_info(HDC hdc, int pf, struct format_info *info)
 {
    memset(info, 0, sizeof(*info));
 
-   if (have_WGL_ARB_pixel_format) {
+   if (GLAD_WGL_ARB_pixel_format) {
       int swapMethod;
 
       info->pfd.dwFlags = 0;
@@ -467,17 +438,28 @@ get_format_info(HDC hdc, int pf, struct format_info *info)
       if (get_pf_attrib(hdc, pf, WGL_STEREO_ARB))
          info->pfd.dwFlags |= PFD_STEREO;
 
+      if (get_pf_attrib(hdc, pf, WGL_DRAW_TO_BITMAP_ARB))
+         info->draw_to_bitmap = true;
+      if (GLAD_WGL_ARB_pbuffer && get_pf_attrib(hdc, pf, WGL_DRAW_TO_PBUFFER_ARB))
+         info->draw_to_pbuffer = true;
+      if (get_pf_attrib(hdc, pf, WGL_SUPPORT_GDI_ARB))
+         info->gdi_drawing = true;
+
       swapMethod = get_pf_attrib(hdc, pf, WGL_SWAP_METHOD_ARB);
       if (swapMethod == WGL_SWAP_EXCHANGE_ARB)
          info->pfd.dwFlags |= PFD_SWAP_EXCHANGE;
       else if (swapMethod == WGL_SWAP_COPY_ARB)
          info->pfd.dwFlags |= PFD_SWAP_COPY;
 
-      info->pfd.iPixelType = get_pf_attrib(hdc, pf, WGL_PIXEL_TYPE_ARB);
-      if (info->pfd.iPixelType == WGL_TYPE_RGBA_ARB)
+      int pixel_type = get_pf_attrib(hdc, pf, WGL_PIXEL_TYPE_ARB);
+      if (pixel_type == WGL_TYPE_RGBA_ARB)
          info->pfd.iPixelType = PFD_TYPE_RGBA;
-      else if (info->pfd.iPixelType == WGL_TYPE_COLORINDEX_ARB)
+      else if (pixel_type == WGL_TYPE_COLORINDEX_ARB)
          info->pfd.iPixelType = PFD_TYPE_COLORINDEX;
+      else if (pixel_type == WGL_TYPE_RGBA_FLOAT_ARB) {
+         info->pfd.iPixelType = PFD_TYPE_RGBA;
+         info->floatComponents = true;
+      }
 
       info->pfd.cColorBits = get_pf_attrib(hdc, pf, WGL_COLOR_BITS_ARB);
       info->pfd.cRedBits = get_pf_attrib(hdc, pf, WGL_RED_BITS_ARB);
@@ -502,6 +484,11 @@ get_format_info(HDC hdc, int pf, struct format_info *info)
       info->numSamples = get_pf_attrib(hdc, pf, WGL_SAMPLES_ARB);
 
       info->transparency = get_pf_attrib(hdc, pf, WGL_TRANSPARENT_ARB);
+
+      if (GLAD_WGL_ARB_framebuffer_sRGB ||
+          GLAD_WGL_EXT_framebuffer_sRGB) {
+         info->srgb = get_pf_attrib(hdc, pf, WGL_FRAMEBUFFER_SRGB_CAPABLE_ARB);
+      }
    }
    else {
       if (!DescribePixelFormat(hdc, pf,
@@ -520,16 +507,12 @@ print_visual_info(HDC hdc, InfoMode mode)
    int numVisuals, numWglVisuals;
    int i;
 
-   wglGetPixelFormatAttribivARB_func =
-      (PFNWGLGETPIXELFORMATATTRIBIVARBPROC)
-      wglGetProcAddress("wglGetPixelFormatAttribivARB");
-
    /* Get number of visuals / pixel formats */
    numVisuals = DescribePixelFormat(hdc, 1,
                                     sizeof(PIXELFORMATDESCRIPTOR), NULL);
    printf("%d Regular pixel formats\n", numVisuals);
 
-   if (have_WGL_ARB_pixel_format) {
+   if (GLAD_WGL_ARB_pixel_format) {
       int numExtVisuals = get_pf_attrib(hdc, 0, WGL_NUMBER_PIXEL_FORMATS_ARB);
       printf("%d Regular + Extended pixel formats\n", numExtVisuals);
       numVisuals = numExtVisuals;
