@@ -30,7 +30,7 @@
 
 #ifdef _WIN32
 /* GL/glext.h is not commonly available on Windows. */
-#include <GL/glew.h>
+#include <glad/glad.h>
 #else
 #include <GL/gl.h>
 #include <GL/glext.h>
@@ -86,6 +86,7 @@ struct options
 
 /** list of known OpenGL versions */
 static const struct { int major, minor; } gl_versions[] = {
+   {4, 6},
    {4, 5},
    {4, 4},
    {4, 3},
@@ -138,5 +139,7 @@ context_flags_string(int mask);
 void
 parse_args(int argc, char *argv[], struct options *options);
 
+void
+print_gpu_memory_info(const char *glExtensions);
 
 #endif /* GLINFO_COMMON_H */
