@@ -100,6 +100,7 @@ struct visual_attribs
    int floatComponents;
    int packedfloatComponents;
    int srgb;
+   int swapMethod;
 };
 
    
@@ -547,6 +548,7 @@ print_screen_info(Display *dpy, int scrnum,
          }
          if (strstr(glxExtensions, "GLX_MESA_query_renderer"))
 	    query_renderer();
+         print_gpu_memory_info(glExtensions);
          printf("OpenGL vendor string: %s\n", glVendor);
          printf("OpenGL renderer string: %s\n", glRenderer);
       } else
@@ -699,6 +701,37 @@ caveat_string(int caveat)
    }
 }
 
+static const char *
+swap_method_verbose_string(int swap_method)
+{
+   switch (swap_method) {
+#ifdef GLX_SWAP_METHOD_OML
+      case GLX_SWAP_EXCHANGE_OML:
+         return "Exchange";
+      case GLX_SWAP_COPY_OML:
+         return "Copy";
+      case GLX_SWAP_UNDEFINED_OML:
+         return "Undefined";
+#endif
+      default:
+         return "Unknown value";
+   }
+}
+
+static char
+swap_method_short_string(int swap_method)
+{
+#ifdef GLX_SWAP_METHOD_OML
+    if (swap_method == GLX_SWAP_EXCHANGE_OML)
+        return 'e';
+    else if (swap_method == GLX_SWAP_COPY_OML)
+        return 'c';
+    else if (swap_method == GLX_SWAP_UNDEFINED_OML)
+        return 'u';
+#endif
+    return '.';
+}
+
 
 static Bool
 get_visual_attribs(Display *dpy, XVisualInfo *vInfo,
@@ -789,6 +822,12 @@ get_visual_attribs(Display *dpy, XVisualInfo *vInfo,
    }
 #endif
 
+#if defined(GLX_OML_swap_method)
+   if (ext && strstr(ext, "GLX_OML_swap_method")) {
+      glXGetConfig(dpy, vInfo, GLX_SWAP_METHOD_OML, &attribs->swapMethod);
+   }
+#endif
+
    return True;
 }
 
@@ -904,6 +943,13 @@ get_fbconfig_attribs(Display *dpy, GLXFBConfig fbconfig,
       glXGetFBConfigAttrib(dpy, fbconfig, GLX_FRAMEBUFFER_SRGB_CAPABLE_EXT, &attribs->srgb);
    }
 #endif
+
+#if defined(GLX_OML_swap_method)
+   if (ext && strstr(ext, "GLX_OML_swap_method")) {
+      glXGetFBConfigAttrib(dpy, fbconfig, GLX_SWAP_METHOD_OML, &attribs->swapMethod);
+   }
+#endif
+
    return True;
 }
 
@@ -959,14 +1005,17 @@ print_visual_attribs_verbose(const struct visual_attribs *attribs,
    else if (attribs->transparentType == GLX_TRANSPARENT_INDEX) {
      printf("    Transparent index=%d\n",attribs->transparentIndexValue);
    }
+#ifdef GLX_SWAP_METHOD_OML
+   printf("    SwapMethod=%s\n", swap_method_verbose_string(attribs->swapMethod));
+#endif
 }
 
 
 static void
 print_visual_attribs_short_header(void)
 {
- printf("    visual  x   bf lv rg d st  colorbuffer  sr ax dp st accumbuffer  ms  cav\n");
- printf("  id dep cl sp  sz l  ci b ro  r  g  b  a F gb bf th cl  r  g  b  a ns b eat\n");
+ printf("    visual  x   bf lv rg d st  colorbuffer  sr ax dp st accumbuffer  ms  sw cav\n");
+ printf("  id dep cl sp  sz l  ci b ro  r  g  b  a F gb bf th cl  r  g  b  a ns b ap eat\n");
  printf("----------------------------------------------------------------------------\n");
 }
 
@@ -996,10 +1045,11 @@ print_visual_attribs_short(const struct visual_attribs *attribs)
           attribs->stencilSize
           );
 
-   printf(" %2d %2d %2d %2d %2d %1d %s\n",
+   printf(" %2d %2d %2d %2d %2d %1d %c  %s\n",
           attribs->accumRedSize, attribs->accumGreenSize,
           attribs->accumBlueSize, attribs->accumAlphaSize,
           attribs->numSamples, attribs->numMultisample,
+          swap_method_short_string(attribs->swapMethod),
           caveat
           );
 }
@@ -1008,9 +1058,9 @@ print_visual_attribs_short(const struct visual_attribs *attribs)
 static void
 print_visual_attribs_long_header(void)
 {
- printf("Vis   Vis   Visual Trans  buff lev render DB ste  r   g   b   a      s  aux dep ste  accum buffer   MS   MS         \n");
- printf(" ID  Depth   Type  parent size el   type     reo sz  sz  sz  sz flt rgb buf th  ncl  r   g   b   a  num bufs caveats\n");
- printf("--------------------------------------------------------------------------------------------------------------------\n");
+ printf("Vis   Vis   Visual Trans  buff lev render DB ste  r   g   b   a      s  aux dep ste  accum buffer   MS   MS  swap        \n");
+ printf(" ID  Depth   Type  parent size el   type     reo sz  sz  sz  sz flt rgb buf th  ncl  r   g   b   a  num bufs mthd caveats\n");
+ printf("-------------------------------------------------------------------------------------------------------------------------\n");
 }
 
 
@@ -1033,7 +1083,7 @@ print_visual_attribs_long(const struct visual_attribs *attribs)
           attribs->blueSize, attribs->alphaSize
           );
 
-   printf("  %c   %c %3d %4d %2d %3d %3d %3d %3d  %2d  %2d %6s\n",
+   printf("  %c   %c %3d %4d %2d %3d %3d %3d %3d  %2d  %2d    %c %6s\n",
           attribs->floatComponents ? 'f' : '.',
           attribs->srgb ? 's' : '.',
           attribs->auxBuffers,
@@ -1042,6 +1092,7 @@ print_visual_attribs_long(const struct visual_attribs *attribs)
           attribs->accumRedSize, attribs->accumGreenSize,
           attribs->accumBlueSize, attribs->accumAlphaSize,
           attribs->numSamples, attribs->numMultisample,
+          swap_method_short_string(attribs->swapMethod),
           caveat
           );
 }
